@@ -1,8 +1,9 @@
 from ._core.descriptors import LazyInstance
 from ._core.injectables import Injectable
 from ._core.module import Mode, Module, Priority, mod
-from ._core.scope import adefine_scope, define_scope
-from ._core.slots import Slot
+from ._core.scope import ScopeFacade as Scope
+from ._core.scope import ScopeKind, adefine_scope, define_scope
+from ._core.slots import SlotKey
 
 __all__ = (
     "Injectable",
@@ -10,7 +11,9 @@ __all__ = (
     "Mode",
     "Module",
     "Priority",
-    "Slot",
+    "Scope",
+    "ScopeKind",
+    "SlotKey",
     "adefine_scope",
     "afind_instance",
     "aget_instance",
