@@ -1,11 +1,13 @@
 from abc import abstractmethod
 from collections.abc import Callable
 from dataclasses import dataclass
-from typing import Protocol, override, runtime_checkable
+from typing import Protocol, runtime_checkable
 
 
 @runtime_checkable
 class Invertible[T](Protocol):
+    __slots__ = ()
+
     @abstractmethod
     def __invert__(self) -> T:
         raise NotImplementedError
@@ -13,8 +15,7 @@ class Invertible[T](Protocol):
 
 @dataclass(repr=False, eq=False, frozen=True, slots=True)
 class SimpleInvertible[T](Invertible[T]):
-    getter: Callable[..., T]
+    callable: Callable[..., T]
 
-    @override
     def __invert__(self) -> T:
-        return self.getter()
+        return self.callable()
