@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import threading
 from abc import ABC, abstractmethod
 from collections import OrderedDict, deque
 from collections.abc import (
@@ -16,7 +17,7 @@ from collections.abc import (
 from contextlib import asynccontextmanager, contextmanager, nullcontext, suppress
 from dataclasses import dataclass, field
 from enum import StrEnum
-from functools import partialmethod, singledispatchmethod, update_wrapper
+from functools import partial, partialmethod, singledispatchmethod, update_wrapper
 from inspect import (
     Signature,
     isasyncgenfunction,
@@ -27,7 +28,6 @@ from inspect import (
 )
 from inspect import signature as inspect_signature
 from logging import Logger, getLogger
-from threading import Lock
 from types import MethodType
 from typing import (
     Any,
@@ -63,11 +63,14 @@ from injection._core.injectables import (
     AsyncCMScopedInjectable,
     CMScopedInjectable,
     Injectable,
+    ScopedInjectable,
+    ScopedSlotInjectable,
     ShouldBeInjectable,
-    SimpleInjectable,
     SimpleScopedInjectable,
     SingletonInjectable,
+    TransientInjectable,
 )
+from injection._core.slots import SlotKey
 from injection.exceptions import (
     ModuleError,
     ModuleLockError,
@@ -222,6 +225,20 @@ class Updater[T]:
     def make_record(self) -> Record[T]:
         return Record(self.injectable, self.mode)
 
+    @classmethod
+    def with_basics(
+        cls,
+        on: TypeInfo[T],
+        /,
+        injectable: Injectable[T],
+        mode: Mode | ModeStr,
+    ) -> Self:
+        return cls(
+            classes=get_return_types(on),
+            injectable=injectable,
+            mode=Mode(mode),
+        )
+
 
 @dataclass(repr=False, frozen=True, slots=True)
 class Locator(Broker):
@@ -283,7 +300,8 @@ class Locator(Broker):
             if injectable.is_locked:
                 continue
 
-            await injectable.aget_instance()
+            with suppress(SkipInjectable):
+                await injectable.aget_instance()
 
     def add_listener(self, listener: EventListener) -> Self:
         self.__channel.add_listener(listener)
@@ -412,7 +430,7 @@ class Module(Broker, EventListener):
         wrapped: Recipe[P, T] | None = None,
         /,
         *,
-        cls: InjectableFactory[T] = SimpleInjectable,
+        cls: InjectableFactory[T] = TransientInjectable,
         ignore_type_hint: bool = False,
         inject: bool = True,
         on: TypeInfo[T] = (),
@@ -420,12 +438,9 @@ class Module(Broker, EventListener):
     ) -> Any:
         def decorator(wp: Recipe[P, T]) -> Recipe[P, T]:
             factory = extract_caller(self.make_injected_function(wp) if inject else wp)
+            injectable = cls(factory)  # type: ignore[arg-type]
             hints = on if ignore_type_hint else (wp, on)
-            updater = Updater(
-                classes=get_return_types(hints),
-                injectable=cls(factory),  # type: ignore[arg-type]
-                mode=Mode(mode),
-            )
+            updater = Updater.with_basics(hints, injectable, mode)
             self.update(updater)
             return wp
 
@@ -445,7 +460,7 @@ class Module(Broker, EventListener):
         def decorator(
             wrapped: Recipe[P, T] | GeneratorRecipe[P, T],
         ) -> Recipe[P, T] | GeneratorRecipe[P, T]:
-            injectable_class: Callable[[Caller[P, Any], str], Injectable[T]]
+            injectable_class: type[ScopedInjectable[Any, T]]
             wrapper: Recipe[P, T] | ContextManagerLikeRecipe[P, T]
 
             if isasyncgenfunction(wrapped):
@@ -465,7 +480,7 @@ class Module(Broker, EventListener):
             hints = on if hint is None else (hint, on)
             self.injectable(
                 wrapper,
-                cls=lambda factory: injectable_class(factory, scope_name),
+                cls=partial(injectable_class, scope_name=scope_name),
                 ignore_type_hint=True,
                 inject=inject,
                 on=hints,
@@ -477,11 +492,8 @@ class Module(Broker, EventListener):
 
     def should_be_injectable[T](self, wrapped: type[T] | None = None, /) -> Any:
         def decorator(wp: type[T]) -> type[T]:
-            updater = Updater(
-                classes=(wp,),
-                injectable=ShouldBeInjectable(wp),
-                mode=Mode.FALLBACK,
-            )
+            injectable = ShouldBeInjectable(wp)
+            updater = Updater.with_basics(wp, injectable, Mode.FALLBACK)
             self.update(updater)
             return wp
 
@@ -526,6 +538,19 @@ class Module(Broker, EventListener):
         )
         return self
 
+    def reserve_scoped_slot[T](
+        self,
+        cls: type[T],
+        /,
+        scope_name: str,
+        *,
+        mode: Mode | ModeStr = Mode.get_default(),
+    ) -> SlotKey[T]:
+        injectable = ScopedSlotInjectable(cls, scope_name)
+        updater = Updater.with_basics(cls, injectable, mode)
+        self.update(updater)
+        return injectable.key
+
     def inject[**P, T](
         self,
         wrapped: Callable[P, T] | None = None,
@@ -558,7 +583,12 @@ class Module(Broker, EventListener):
         threadsafe: bool = ...,
     ) -> AsyncInjectedFunction[P, T]: ...
 
-    def make_injected_function(self, wrapped, /, threadsafe=False):  # type: ignore[no-untyped-def]
+    def make_injected_function[**P, T](
+        self,
+        wrapped: Callable[P, T],
+        /,
+        threadsafe: bool = False,
+    ) -> InjectedFunction[P, T]:
         metadata = InjectMetadata(wrapped, threadsafe)
 
         @metadata.task
@@ -567,7 +597,7 @@ class Module(Broker, EventListener):
             self.add_listener(metadata)
 
         if iscoroutinefunction(wrapped):
-            return AsyncInjectedFunction(metadata)
+            return AsyncInjectedFunction(metadata)  # type: ignore[arg-type, return-value]
 
         return SyncInjectedFunction(metadata)
 
@@ -605,7 +635,11 @@ class Module(Broker, EventListener):
         default: None = ...,
     ) -> T | None: ...
 
-    async def aget_instance(self, cls, default=None):  # type: ignore[no-untyped-def]
+    async def aget_instance[T, Default](
+        self,
+        cls: InputType[T],
+        default: Default | None = None,
+    ) -> T | Default | None:
         try:
             return await self.afind_instance(cls)
         except (KeyError, SkipInjectable):
@@ -625,7 +659,11 @@ class Module(Broker, EventListener):
         default: None = ...,
     ) -> T | None: ...
 
-    def get_instance(self, cls, default=None):  # type: ignore[no-untyped-def]
+    def get_instance[T, Default](
+        self,
+        cls: InputType[T],
+        default: Default | None = None,
+    ) -> T | Default | None:
         try:
             return self.find_instance(cls)
         except (KeyError, SkipInjectable):
@@ -649,7 +687,13 @@ class Module(Broker, EventListener):
         cache: bool = ...,
     ) -> Awaitable[T | None]: ...
 
-    def aget_lazy_instance(self, cls, default=None, *, cache=False):  # type: ignore[no-untyped-def]
+    def aget_lazy_instance[T, Default](
+        self,
+        cls: InputType[T],
+        default: Default | None = None,
+        *,
+        cache: bool = False,
+    ) -> Awaitable[T | Default | None]:
         if cache:
             return alazy(lambda: self.aget_instance(cls, default))
 
@@ -675,7 +719,13 @@ class Module(Broker, EventListener):
         cache: bool = ...,
     ) -> Invertible[T | None]: ...
 
-    def get_lazy_instance(self, cls, default=None, *, cache=False):  # type: ignore[no-untyped-def]
+    def get_lazy_instance[T, Default](
+        self,
+        cls: InputType[T],
+        default: Default | None = None,
+        *,
+        cache: bool = False,
+    ) -> Invertible[T | Default | None]:
         if cache:
             return lazy(lambda: self.get_instance(cls, default))
 
@@ -765,8 +815,6 @@ class Module(Broker, EventListener):
 
         self.unlock().init_modules(*modules)
 
-        del module, modules
-
         @contextmanager
         def cleaner() -> Iterator[Self]:
             yield self
@@ -949,7 +997,7 @@ class InjectMetadata[**P, T](Caller[P, T], EventListener):
 
     def __init__(self, wrapped: Callable[P, T], /, threadsafe: bool) -> None:
         self.__dependencies = Dependencies.empty()
-        self.__lock = Lock() if threadsafe else nullcontext()
+        self.__lock = threading.Lock() if threadsafe else nullcontext()
         self.__owner = None
         self.__tasks = deque()
         self.__wrapped = wrapped
